# Berechnungsroutiene des Julianischen Tages aus dem brgerlichen Datum und umgekehrt.
# Version: 14.06.1999 
# Diese Library ist Bestandteil des AstroIx Programmpaketes
# Copyright (c) 1997-99 Xaman Ek Xols
# Autor: Wolfgang Pawlowski

proc getlocaltime {} {
	global in_datum in_zeit
	set in_datum(Jahr) [clock format [clock seconds] -format "%Y"]
	set in_datum(Monat) [clock format [clock seconds] -format "%m"]
	regsub -all "(^| )0" $in_datum(Monat) "\\1" in_datum(Monat)
	set in_datum(Tag) [clock format [clock seconds] -format "%d"]
	regsub -all "(^| )0" $in_datum(Tag) "\\1" in_datum(Tag)
	set in_zeit(Stunde) [clock format [clock seconds] -format "%H"]
	regsub -all "(^| )0" $in_zeit(Stunde) "\\1" in_zeit(Stunde)
	set in_zeit(Minute) [clock format [clock seconds] -format "%M"]
	regsub -all "(^| )0" $in_zeit(Minute) "\\1" in_zeit(Minute)
	set in_zeit(Sekunde) [clock format [clock seconds] -format "%S"]
	regsub -all "(^| )0" $in_zeit(Sekunde) "\\1" in_zeit(Sekunde)
}

proc out>in_time {} {
	global in_datum in_zeit out_datum out_zeit
	set in_datum(Jahr)   $out_datum(Jahr)
	set in_datum(Monat)  $out_datum(Monat)
	set in_datum(Tag)    $out_datum(Tag)
	set in_zeit(Stunde)  $out_zeit(Stunde)
	set in_zeit(Minute)  $out_zeit(Minute)
	set in_zeit(Sekunde) $out_zeit(Sekunde)
}

proc Zeitumwandlung {x} {
	global out_zeit
	if {$x < 0} {
		set x [expr $x + 24.0]
	}
	set x  [expr (($x / 24.0) - double(int($x / 24.0))) * 24.0]
	set out_zeit(Stunde) [expr int($x)]
	set out_zeit(Minute) [expr  int(60 * ($x - $out_zeit(Stunde)))]
	set out_zeit(Sekunde) [expr 60 * (60 * ($x - $out_zeit(Stunde)) - $out_zeit(Minute))]
	if {$out_zeit(Sekunde) > 59.999} {
		set out_zeit(Minute) [expr $out_zeit(Minute) + 1]
		set out_zeit(Sekunde) 0
	}
	if {$out_zeit(Minute) == 60} {
		set out_zeit(Stunde) [expr $out_zeit(Stunde) + 1]
		set out_zeit(Minute) 0
	}
}

proc Schaltjahrtest {jahr} {
	set limit 355
	if {$jahr < 0 } {
		if {[expr fmod($jahr + 1, 4)] == 0.0} {
			set limit 366
		}
	} else {
		if {[expr fmod($jahr, 4)] != 0.0} {
			set limit 365
		} else {
			set limit 366
			if {$jahr >= 1582} {
				set limit 365
				if {[expr fmod($jahr, 100)] != 0.0} {
					set limit 366
				} else {
					if {[expr fmod($jahr, 400)] == 0.0} {
						set limit 366
					}
				}
			}
		}
	}
	return $limit
}

proc jd {} {
	global in_datum in_zeit
	array set month { 1 {13} 2 {14} 3 {3} 4 {4} 5 {5} 6 {6} 7 {7} 8 {8} 9 {9} 10 {10} 11 {11} 12 {12}}
	array set Tage    { 1 {31} 2 {28} 3 {31} 4 {30} 5 {31} 6 {30} 7 {31} 8 {31} 9 {30} 10 {31} 11 {30} 12 {31} 13 {29}}
	set oki true
	set Tagr [expr double($in_datum(Tag)) + (double($in_zeit(Stunde)) + double($in_zeit(Minute)) / 60 \
                + double($in_zeit(Sekunde)) / 3600) / 24]
	if {($in_datum(Jahr) < -4713) || ($in_datum(Jahr) > 1600000000)} { 
		set oki false
		error "Die Eingabe liegt nicht im Berechnungsbereich !"
	} else {
		if {$in_datum(Jahr) == 0} {
			set oki false
			error "Das Jahr 0 gibt es nicht !"
		}
		if {$oki} {
			if {$in_datum(Monat) < 13} {
				set limit $Tage($in_datum(Monat))
				if {$in_datum(Monat) == 2 } {
					if {[Schaltjahrtest $in_datum(Jahr)] == 365} {
						set limit 28
					} else {
						set limit 29
					}
				}
			} else {
				set oki false
				error "Dieser Monat existiert nicht !"
			}
		}
		if {$oki} {
			if {($in_datum(Tag) >= 1) && ($in_datum(Tag) <= $limit)} {
				set td [expr double($in_datum(Jahr)) + double($in_datum(Monat)) / 100.0 + $Tagr / 10000.0]
				if {($td > 1582.1004) && ($td < 1582.1015)} {
					set oki false
					error "Die Tage vom 5.10 bis 14.10.1582 gibt es nicht!"
				}  
			} else {
				set oki false
				error "Dieser Tag existiert nicht !"
			}
		}
		set HJahr double($in_datum(Jahr))
		if {$in_datum(Jahr) < 0} {
			set HJahr [expr $HJahr + 1]
		}
		set M1 $month($in_datum(Monat))
		if {$M1 > 12} { 
			set HJahr [expr $HJahr - 1]
		}
		set Summe 0
		set Summe [expr int($HJahr/100)]
		set Summe [expr 2 - $Summe + int($Summe / 4)]
		if {[expr (double($in_datum(Jahr)) + double($in_datum(Monat)) / 100.0 + $Tagr / 10000.0)] < 1582.1005} { 
			set Summe 0
		}
		if {$HJahr < 0} {
			set Summe [expr -1 * int(-365.25 * $HJahr + 0.75) + int(30.6001 * ($M1 + 1)) + $Tagr + 1720994.5] 
		} else {
			set Summe [expr $Summe + int(365.25 * $HJahr) + int(30.6001 * ($M1 + 1)) + $Tagr + 1720994.5] 
		}
		return $Summe 
	}
}

proc buergerchen {julia} {
	global out_datum 
	if {$julia > 6E+11} {
		error "Auerhalb des berechenbaren Zeitraumes! Bei JD = 6*10^11 ist Schlu."
	}
	set summe $julia
	if {$julia >= 0} {
		if {$julia <= 1721117.0} {
			set jr [expr int(($julia - 1720995.0)/365.25)]
			set jm 0
			while {$jm < 3} {
				set jt [expr int($julia - 1720995.0 + int(-1 * $jr * 365.25 + 0.75))]
				set jm [expr int($jt/30.6001) - 1]
				if {$jm < 3} { 
					set jr [expr $jr - 1]
				}
			}
			set jt [expr $jt - int(30.6001 * ($jm + 1))]
			if {$jm > 12} {
				set jm [expr $jm - 12]
				set jr [expr $jr + 1]
			}
			if {$jr <= 0} {
				set jr [expr $jr - 1]
				set jz 1
			} else {
				set jz 2
			}
		} else {
			set jz 2
			set a [expr int(($julia - 1720995.0) / 365.25)]
			set b 0
			set okim -1
			if {$julia < 2299161.0} {
				set okim 0
			} else {
				while {$okim != 0} {
					set b [expr int($a / 100)]
					set b [expr 2 - $b + int($b / 4)]
					set a1 [expr int(($julia - 1720995.0 - $b) / 365.25)]
					if {$a1 != $a} {
						set a $a1
					} else {
						set okim 0
					}
				}
			}
			set jm 0
			while {$jm < 3} {
				set jr $a
				set jt [expr int($julia - 1720995.0 - $b - int($a * 365.25))]
				set jm [expr int($jt/30.6001) - 1]
				set jt [expr $jt - int(30.6001 *($jm + 1))]
				if {$jm < 3} {
					set a [expr $a - 1]
					set b 0
					if {$julia >= 2299161.0} {
						set b [expr int($a / 100)]
						set b [expr 2 - $b + int($b / 4)]
					}
				}
			}
			if {$jm > 12} {
				set jm [$jm - 12]
				set jr [$jr + 1]
			}
			if {$jr <= 0} {
				set jz 1
				set jr [expr $jr - 1]
			}
		}
		if {$jz == 1} {
			set out_datum(Jahr) [expr $jr + 1]
		} else {
			set out_datum(Jahr) [expr abs($jr)]
		}
		set out_datum(Monat) $jm
		set out_datum(Tag) $jt
	} else {
		error "Ein Julianisches Datum kleiner 0 ist nicht definiert!"
	}
}

proc Buerger {julia} {
	global out_zeit 
	set rest [expr $julia - int($julia)]
	if {$rest < 0.5} {
		buergerchen [expr int($julia)]
		if {$rest == 0} {
			set out_zeit(Stunde)  12
			set out_zeit(Minute)  0
			set out_zeit(Sekunde) 0
		} else {
			set oh [expr int($rest * 24.0)]
			set oh2 [expr $oh + 12]
			set out_zeit(Stunde)  $oh2
			set om [expr int((($rest * 24.0) - $oh) * 60.0)]
			set out_zeit(Minute)  $om
			set os [expr int((((($rest * 24.0) - $oh) * 60.0) - $om) * 60)]
			set out_zeit(Sekunde) $os 
		}
	} else {
		buergerchen [expr int($julia) + 1]
		set oh [expr int($rest * 24.0)]
		set oh2 [expr $oh - 12]
		set out_zeit(Stunde)  $oh2
		set om [expr int((($rest * 24.0) - $oh) * 60.0)]
		set out_zeit(Minute)  $om
		set os [expr int((((($rest * 24.0) - $oh) * 60.0) - $om) * 60)]
		set out_zeit(Sekunde) $os 
	}
}

proc Dynamische_Zeit {} {
  global in_datum in_zeit
  array set ddt {1620 {1240} 1622 {1150} 1624 {1060} 1626 {980} 1628 {910} 1630 {850} 1632 {790} 1634 {740} 1636 {700} 1638 {650} \
                       1640 {620} 1642 {580} 1644 {550} 1646 {530} 1648 {500} 1650 {480} 1652 {460} 1654 {440} 1656 {420} 1658 {400} \
                       1660 {370} 1662 {350} 1664 {330} 1666 {310} 1668 {280} 1670 {260} 1672 {240} 1674 {220} 1676 {200} 1678 {180} \
                       1680 {160} 1682 {140} 1684 {130} 1686 {120} 1688 {110} 1690 {100} 1692 {90} 1694 {90} 1696 {90} 1698 {90} \
                       1700 {90} 1702 {90} 1704 {90} 1706 {90} 1708 {100} 1710 {100} 1712 {100} 1714 {100} 1716 {100} 1718 {110} \
                       1720 {110} 1722 {110} 1724 {110} 1726 {110} 1728 {110} 1730 {110} 1732 {110} 1734 {120} 1736 {120} 1738 {120} \
                       1740 {120} 1742 {120} 1744 {130} 1746 {130} 1748 {130} 1750 {130} 1752 {140} 1754 {140} 1756 {140} 1758 {150} \
                       1760 {150} 1762 {150} 1764 {150} 1766 {160} 1768 {160} 1770 {160} 1772 {160} 1774 {160} 1776 {170} 1778 {170} \  
                       1780 {170} 1782 {170} 1784 {170} 1786 {170} 1788 {170} 1790 {170} 1792 {160} 1794 {160} 1796 {150} 1798 {140} \
                       1800 {137} 1802 {131} 1804 {127} 1806 {125} 1808 {125} 1810 {125} 1812 {125} 1814 {125} 1816 {125} 1818 {123} \
                       1820 {120} 1822 {114} 1824 {106} 1826 {96} 1828 {86} 1830 {75} 1832 {66} 1834 {60} 1836 {57} 1838 {56} \
                       1840 {57} 1842 {59} 1844 {62} 1846 {65} 1848 {68} 1850 {71} 1852 {73} 1854 {75} 1856 {77} 1858 {78} \
                       1860 {79} 1862 {75} 1864 {64} 1866 {54} 1868 {29} 1870 {16} 1872 {-10} 1874 {-27} 1876 {-36} 1878 {-47} \
                       1880 {-54} 1882 {-52} 1884 {-55} 1886 {-56} 1888 {-58} 1890 {-59} 1892 {-62} 1894 {-64} 1896 {-61} 1898 {-47} \
                       1900 {-27} 1902 {0} 1904 {26} 1906 {54} 1908 {77} 1910 {105} 1912 {134} 1914 {160} 1916 {182} 1918 {202} \
                       1920 {212} 1922 {224} 1924 {235} 1926 {239} 1928 {243} 1930 {240} 1932 {239} 1934 {239} 1936 {237} 1938 {240} \
                       1940 {243} 1942 {253} 1944 {262} 1946 {273} 1948 {282} 1950 {291} 1952 {300} 1954 {307} 1956 {314} 1958 {322} \
                       1960 {331} 1962 {340} 1964 {350} 1966 {365} 1968 {383} 1970 {402} 1972 {422} 1974 {445} 1976 {465} 1978 {485} \
                       1980 {505} 1982 {522} 1984 {538} 1986 {549} 1988 {558} 1990 {569} 1992 {583} 1994 {605} 1996 {627} 1998 {649} \
                       2000 {670} 2002 {696} 2004 {722} 2006 {748} 2008 {774} 2010 {800}}
  set jahr [expr double($in_datum(Jahr))]
  set monat [expr double($in_datum(Monat))]
  set tag [expr double($in_datum(Tag))]
  set z [expr $jahr + $monat / 12 + $tag / 365.25]
  set t [expr ($z - 2000)  / 100]   
  if {$jahr < 948} { 
    set out [expr 2715.6 + 573.36 * $t + 46.5 * $t * $t]
  }
  if {($jahr >= 948) && ($ahr < 1620)} {
    set out [expr 50.6 + 67.5 * $t + 22.5 * $t * $t]
  }
  if {($jahr >= 1620 && $jahr < 2010)}{
    set i [expr int($z / 2 + 0.5) * 2]
    set n [expr $z - (1620 + (2 * $i))]
    if {$jahr == 1620} {
      set i 1620
      set n [expr $z - 1622]
    }
    set y1 [expr $ddt([expr $i - 1]) / 10]
    set y2 [expr $ddt($i) / 10]
    set y3 [expr $ddt([expr $i + 1]) / 10]
    set a [expr $y2 - $y1]
    set b [expr $y3 - $y2]
    set c [expr $b - $a]
    set y [expr $y2 + $n / 2 * ($a + $b + $n * $c)]
    set out $y
  }
  if {$jahr >= 2010} {
    error "Gerechte Werte sind nicht mehr interpolierbar!"
  } 
  set out [expr $stunde + $minute / 60 + $sekunde / 3600 + $out / 3600]
#      Zeitumwandlung(out)
  return $out
}

proc Mittlere_Sternzeit {jdv zonediff} {
	set t [expr ($jdv - 2451545.0) / 36525.0]
	set st [expr 280.46061837 + 360.98564736629 * ($jdv - 2451545.0) + 0.000387933 * $t * $t]
	set st [expr $st - (($t * $t * $t) / 38710000)]
	if {$st < 0} {
		set l1 [expr int($st / 360.0 - 0.5) * -1]
		set st [expr $st +  (($l1 + 1) * 360)]
	}
	if {$st > 360.0} {
		set l1 [expr int($st / 360.0 + 0.5)]
		set st [expr $st - ($l1 * 360)]
	}
	set st [expr $st / 15]
	set st [expr $st + $zonediff]
#  Zeitumwandlung(st) ist hier noch notwendig
	return $st
}

proc Wahre_Sternzeit {jdv zonediff phi eps} {
	global const_pi  
	set st [Mittlere_Sternzeit $jdv $zonediff]
	set diffst [expr cos($const_pi * $eps / 180.0) * $phi / 15.0]
	set wast [expr $st + ($diffst / 3600.0)]
#    Zeitumwandlung(wast) ist hier noch notwendig
	return $wast
}

proc Osterdatum {} {
	global in_datum out_datum
	set jahr $in_datum(Jahr)
	if {$jahr >= 1583} {
		set a [expr fmod($jahr, 19)]
		set b [expr $jahr / 100] 
		set c [expr fmod($jahr, 100)]
		set f [expr ($b + 8) / 25]
		set g [expr ($b - $f + 1) / 3] 
		set h [expr 19 * $a + $b - int($b / 4) - $g + 15]
		set h [expr fmod($h, 30)]
		set l [expr 32 + 2 * fmod($b, 4) + 2 * ($c / 4) - $h - fmod($c, 4)]
		set l [expr fmod($l, 7)]
		set m [expr ($a + 11 * $h + 22 * $l) / 451]
		set ll [expr $h + $l - 7 * $m + 114]
		set n [expr $ll / 31]
		set p [expr fmod($ll, 31) + 1]
		set out_datum(Tag) $n 
		set out_datum(Monat) $p
		set out_datum(Jahr) $jahr
	} else { 
		set d [expr (19 * fmod($jahr, 19) + 15)]
		set d [expr fmod($d, 30)]
		set e [expr 2 * fmod($jahr, 4) + 4 * fmod($jahr, 7) - $d + 34]
		set e [expr $d + fmod($e, 7) + 114]
		set f [expr $e / 31]
		set g [expr fmod($e, 31) + 1]
		set out_datum(Tag) $f 
		set out_datum(Monat) $g
		set out_datum(Jahr) $jahr
	}
}

proc Wochentag {jdv} {
	set jdv [expr $jdv + 1.5]
	set wota [expr int(fmod(($jdv), 7))]
	return $wota
}
