# Umrechnungsroutiene verschiedener Koordinatensysteme.
# Version: 27.11.1997
# Diese Library ist Bestandteil des AstroIx Programmpaketes
# Copyright (c) 1997 Xaman Ek Xols
# Autor: Wolfgang Pawlowski

# Bentigt wird die Breite aus 'Heimat', Hhe, Azimut und die Sternzeit (in_zeit)
# Ausgabe in Rektaszension und Deklination.
proc HorizontAequator { hoehe azimut } {
  global home 
  global in_zeit
  global const_pi  
  global a
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set gb [expr $c0 * $home(Breite)]
  set h  [expr $c0 * $hoehe]
  set az [expr $c0 * $azimut]
  set st [expr $in_zeit(Stunde) + $in_zeit(Minute) / 60.0 + $in_zeit(Sekunde) / 3600.0]
  set st [expr $c1 * $st]
  set cosh [expr cos($h)]
  set sinh [expr sin($h)]
  set cosgb [expr cos($gb)]
  set singb [expr sin($gb)]
  set cosaz [expr cos($az)]
  set x [expr -1 * $cosh * sin($az)]
  set y [expr $sinh * $cosgb - $cosh * $cosaz * $singb]
  set z [expr $sinh * $singb + $cosh * $cosaz * $cosgb]
  set t [expr atan($x / $y)]
  if {$y < 0} {
    set t [expr $t + $const_pi]
  } else {
    if {$x < 0} {
      set t [expr $t + $pi2]
    }
  }
  set de [expr atan($z / sqrt($x * $x + $y * $y))]
  set ra [expr $st - $t]
  if {$ra < 0} {
    set ra [expr $ra + $pi2]
  }
  if {$ra > $pi2} {
    set ra [expr $ra - $pi2]
  }
  set ra [expr $ra / $c1]
  if {$de >= 0} {
    set de [expr $de / $c0]
  } else {
     set de [expr -1 * int(-1 * $de / $c0)]
  }
  return [concat $ra " " $de]
}

# Bentigt wird die Rektaszension, Deklination, die Breite aus 'Heimat' und die Sternzeit (in_zeit)
# Ausgabe in Azimut und Hhe.
proc AequatorHorizont {ra de } {
  global home 
  global in_zeit
  global const_pi  
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set gb [expr $c0 * $home(Breite)]
  set de [expr $de * $c0]
  set ra [expr $ra * $c1]
  set st [expr $in_zeit(Stunde) + $in_zeit(Minute) / 60.0 + $in_zeit(Sekunde) / 3600.0]
  set st [expr $st * $c1]
  set t [expr $st - $ra]
  set cosde [expr cos($de)]
  set sinde [expr sin($de)]
  set cosgb [expr cos($gb)]
  set singb [expr sin($gb)]
  set cost [expr cos($t)]
  set x [expr -1 * $cosde * sin($t)]
  set y [expr $sinde * $cosgb - $cosde * $cost * $singb]
  set z [expr $sinde * $singb + $cosde * $cost * $cosgb]
  set az [expr atan($x / $y)]
  if {$y < 0} {
    set az [expr $az + $const_pi]
  } else {
    if {$x < 0} {
      set az [expr $az + $const_pi]
    }
  }
  set h [expr atan($z / sqrt($x * $x + $y * $y))]
  set az [expr $az / $c0]
  set h [expr $h / $c0]
  return [concat $az " " $h]
}

# Bentigt wird der Julianischer Tag, die eklitikale Lnge und Breite
# Ausgabe in Rektaszension und Deklination.
proc EkliptikAequator {jdv l b} {
  global const_pi  
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set t [expr ($jdv - 2415020.0) / 36525.0]
  set ep [expr 23.452 - 0.01301 * $t - 1.6E-6 * $t * $t + 5E-7 * $t * $t * $t]
  set ep [expr $c0 * $ep]
  set l [expr $c0 * $l]
  set b [expr $c0 * $b]
  set cosb [expr cos($b)]
  set sinb [expr sin($b)]
  set cosep [expr cos($ep)]
  set sinep [expr sin($ep)]
  set sinl [expr sin($l)]
  set x [expr $cosb * cos($l)]
  set y [expr $cosb * $sinl * $cosep - $sinb * $sinep]
  set z [expr $cosb * $sinl * $sinep + $sinb * $cosep]
  set ra [expr atan($y / $x)]
  if {$x < 0} {
    set ra [expr $ra + $const_pi]
  } else {
    if {$y < 0} {
      set ra [expr $ra + $pi2]
    }
  }
  set de [expr atan($z / sqrt($x * $x + $y * $y))]
  set ra [expr $ra / $c0 / 15.0]
  set de [expr $de / $c0]
  return [concat $ra " " $de]
}

# Bentigt wird der Julianischer Tag, Rektaszension und Deklination. 
# Ausgabe der eklitikale Lnge und Breite.
proc AequatorEkliptik {jdv ra de} {
  global const_pi  
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set ra [expr $c1 * $ra]
  set de [expr $c0 * $de]
  set t  [expr ($jdv - 2415020.0) / 36525.0]
  set ep [expr 23.452 + $t *(-0.01301 + $t * (-1.6E-6 + 5E-7 * $t))]
  set ep [expr $c0 * $ep]
  set cosde [expr cos($de)]
  set sinde [expr sin($de)]
  set cosep [expr cos($ep)]
  set sinep [expr sin($ep)]
  set sinra [expr sin($ra)]
  set x [expr $cosde * cos($ra)]
  set y [expr $cosde * $sinra * $cosep + $sinde * $sinep]
  set z [expr $sinde * $cosep - $cosde * $sinra * $sinep]
  set l  [expr atan($y / $x)]
  if {$x < 0} {
    set l [expr $l + $const_pi]
  } else {
    if {$y < 0} {
      set l [expr $l + $pi2]
    }
  }
  set b [expr atan($z / sqrt($x * $x + $y * $y))]
  set l [expr $l / $c0]
  set b [expr $b / $c0]
  return [concat $l " " $b]
}

# Bentigt wird der Julianischer Tag, die galaktische Lnge und Breite
# Ausgabe in Rektaszension und Deklination.
proc Galaktik2Aequator {jdv l b} {
  global const_pi
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set l [expr $c0 * $l]
  set b [expr $c0 * $b]
  set t [expr ($jdv - 2433282.0) / 365.25]
  set e [expr 62.6 + 0.005438 * $t]
  set d [expr 282.25 + 0.012187 * $t]
  set f [expr 33.0 - 0.001344 * $t]
  set e [expr $c0 * $e]
  set d [expr $c0 * $d]
  set f [expr $c0 * $f]
  set cosb [expr cos($b)]
  set sinb [expr sin($b)]
  set cose [expr cos($e)]
  set sine [expr sin($e)]
  set sinlf [expr sin($l - $f)]
  set x [expr $cosb * cos($l - $f)]
  set y [expr $cosb * $sinlf * $cose - $sinb * $sine]
  set z [expr $cosb * $sinlf * $sine + $sinb * $cose]
  set ra [expr atan($y / $x)]
  if {$x < 0} {
    set ra [expr $ra + $const_pi]
  } else {
    if {$y < 0} {
      set ra [expr $ra + $pi2]
    }
  }
  set ra [expr $ra + $d]
  if {$ra < 0} {
    set ra [expr $ra + $pi2]
  }
  if {$ra > $pi2} {
    set ra [expr $ra - $pi2]
  }
  set de [expr atan($z / sqrt($x * $x + $y * $y))]
  set ra [expr $ra / $c0 / 15.0]
  set de [expr $de / $c0]
  return [concat $ra " " $de]  
}

# Bentigt wird der Julianischer Tag, Rektaszension und Deklination. 
# Ausgabe der galaktischen Lnge und Breite.
proc AequatorGalaktik2 {jdv ra de} {
  global const_pi
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set ra [expr $c1 * $ra]
  set de [expr $c0 * $de]
  set t [expr ($jdv - 2433282.0) / 365.25]
  set e [expr 62.6 + 0.005438 * $t]
  set d [expr 282.25 + 0.012187 * $t]
  set f [expr 33.0 - 0.001344 * $t]
  set e [expr $c0 * $e]
  set d [expr $c0 * $d]
  set f [expr $c0 * $f]
  set cosde [expr cos($de)]
  set sinde [expr sin($de)]
  set cose [expr cos($e)]
  set sine [expr sin($e)]
  set sinrad [expr sin($ra - $d)]
  set x [expr $cosde * cos($ra - $d)]
  set y [expr $cosde * $sinrad * $cose + $sinde * $sine]
  set z [expr $sinde * $cose - $cosde * $sinrad * $sine]
  set l [expr atan($y / $x)]
  if {$x < 0} {
    set l [expr $l + $const_pi]
  } else {
    if {$y < 0} {
      set l [expr $l + $pi2]
    }
  }
  set l [expr $l + $f]
  if {$l < 0} {
    set l [expr $l + $pi2]
  }
  if {$l > $pi2} {
    set l [expr $l - $pi2]
  }
  set b [expr atan($z / sqrt($x * $x + $y * $y))]
  set l [expr $l / $c0]
  set b [expr $b / $c0]
  return [concat $l " " $b]
}

# Bentigt wird die galaktischen LngeI und BreiteI.
# Ausgabe der galaktischen LngeII und BreiteII.
proc Galaktik1Galaktik2 {li bi} {
  global const_pi
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set li [expr $c0 * $li]
  set bi [expr $c0 * $bi]
  set e  [expr 1.4866 * $c0]
  set d  [expr 257.65 * $c0]
  set f  [expr 289.9497 * $c0]
  set cosbi [expr cos($bi)]
  set sinbi [expr sin($bi)]
  set cose [expr cos($e)]
  set sine [expr sin($e)]
  set sinlid [expr sin($li - $d)]
  set x [expr $cosbi * cos($li - $d)]
  set y [expr $cose * $cosbi * $sinlid - $sinbi * $sine]
  set z [expr $cose * $sinlid * $sine + $sinbi * $cose]
  set l2 [expr atan($y / $x)]
  if {$x < 0} {
    set l2 [expr $l2 + $const_pi]
  } else {
    if {$y < 0} {
      set l2 [expr $l2 + $pi2]
    }
  }
  set l2 [expr $l2 + $f]
  if {$l2 < 0} {
    set l2 [expr $l2 + $pi2]
  }
  if {$l2 > $pi2} {
    set l2 [expr $l2 - $pi2]
  }
  set b2 [expr atan($z / sqrt($x * $x + $y * $y))]
  set l2 [expr $l2 / $c0]
  set b2 [expr $b2 / $c0]
  return [concat $l2 " " $b2]
}

# Bentigt wird die galaktischen LngeII und BreiteII
# Ausgabe der galaktischen LngeI und BreiteI.
proc Galaktik2Galaktik1 {l2 b2} {
  global const_pi
  set c0 [expr $const_pi / 180.0]
  set c1 [expr $c0 * 15] 
  set pi2 [expr $const_pi * 2.0]
  set l2 [expr $c0 * $l2]
  set b2 [expr $c0 * $b2]
  set e [expr 1.4866 * $c0]
  set d [expr 257.65 * $c0]
  set f  [expr 289.9497 * $c0]
  set cosb2 [expr cos($b2)]
  set sinb2 [expr sin($b2)]
  set cose [expr cos($e)]
  set sine [expr sin($e)]
  set sinl2f [expr sin($l2 - $f)]
  set x [expr $cosb2 * cos($l2 - $f)]
  set y [expr $sine * $sinb2 + $cose * $cosb2 * $sinl2f]
  set z [expr $cose * $sinb2 - $sine * $cosb2 * $sinl2f]
  set li [expr atan($y / $x)]
  if {$x < 0} {
    set li [expr $li + $const_pi]
  } else {
    if {$y < 0} {
      set li [expr $li + $pi2]
    }
  }
  set li [expr $li + $d]
  if {$li < 0} {
    set li [expr $li + $pi2]
  }
  if  {$li > $pi2} {
    set li [expr $li - $pi2]
  }
  set bi [expr atan($z / sqrt($x * $x + $y * $y))]
  set li [expr $li / $c0]
  set bi [expr $bi / $c0]
  return [concat $li " " $bi]
}
    

    